"""Tools packages."""

import os

from arduinozore.settings import DEVICE_CONFIG_FOLDER
from arduinozore.settings import path
from serial.tools import list_ports
from yaml import safe_load


def get_arduinos():
    """Return connected arduinos devices."""
    serials = list(list_ports.comports())
    return [s for s in serials if 'Arduino' in s.description or (
        s.manufacturer is not None and 'Arduino' in s.manufacturer)]


def get_arduino(name):
    """Return arduino by name."""
    arduinos = get_arduinos()
    arduino = next(
        (arduino for arduino in arduinos if name in arduino.device), None)
    return arduino


def load_config(config_file):
    """Load port list from specific config file."""
    with open(config_file, 'r') as f:
        datas = safe_load(f)
    ports = {k: datas['ports'][k]
             for k in datas['ports'] if len(datas['ports'][k]) is not 0}
    return (datas['device_name'], ports)


def load_config_from_arduino(arduino):
    """Load port list corresponding to arduino if it exists."""
    filenames = os.listdir(DEVICE_CONFIG_FOLDER)
    config_name = get_config_name(arduino)
    config_file = path(DEVICE_CONFIG_FOLDER, config_name)
    if config_name in filenames:
        return load_config(config_file)
    else:
        return (None, None)


def get_config_name(arduino):
    """Return config filename for arduino."""
    config_name = str(arduino.vid)
    config_name += str(arduino.pid)
    config_name += str(arduino.serial_number)
    config_name += ".yaml"
    return config_name



"""Arduinozore module."""

import argparse
import os
import socket
import sys
from shutil import copyfile
from shutil import get_terminal_size
from shutil import rmtree

import tornado.httpserver
import tornado.ioloop
import tornado.web
import tornado.websocket
from arduinozore.handlers.error404handler import My404Handler
from arduinozore.handlers.serialManager import SerialManager
from arduinozore.handlers.ws import WSHandler
from arduinozore.settings import ARDUINO_CODE_FILE_NAME
from arduinozore.settings import ARDUINO_FILE_PATH
from arduinozore.settings import CERT_FOLDER
from arduinozore.settings import CERT_INSTALLER_PATH
from arduinozore.settings import CONFIG_FOLDER
from arduinozore.settings import PORT
from arduinozore.settings import SSL_PORT
from arduinozore.settings import STATIC_INSTALLER_PATH
from arduinozore.settings import STATIC_PATH
from arduinozore.settings import path
from arduinozore.settings import settings
from arduinozore.settings import ssl_opts
from arduinozore.urls import url_pattern
from pyfiglet import Figlet

STATIC_CMD = "chmod +x " + STATIC_INSTALLER_PATH + " && " + STATIC_INSTALLER_PATH
CERT_CMD = "chmod +x " + CERT_INSTALLER_PATH + " && " + CERT_INSTALLER_PATH


def main():
    """Catch main function."""
    p = argparse.ArgumentParser(
        description="Arduinozore server", prog="arduinozore")
    p.add_argument('-hp',
                   '--http_port',
                   type=int,
                   help='Server http port. Default ' + str(PORT))
    p.add_argument('-hsp',
                   '--https_port',
                   type=int,
                   help='Server https port. Default ' + str(SSL_PORT) + '. Used for sockets too.')
    p.add_argument('-a',
                   '--arduino',
                   type=str,
                   metavar='path',
                   help='Path where arduino source code will be generated.')
    p.add_argument('--newconfig',
                   action="store_true",
                   help='Delete actual config and make a new one. Warning.')
    args = p.parse_args()

    if args.arduino:
        copy_arduino_code(args.arduino)

    if args.newconfig:
        if os.path.exists(CONFIG_FOLDER):
            rmtree(CONFIG_FOLDER)

    http_port = PORT if args.http_port is None else args.http_port
    ssl_port = SSL_PORT if args.https_port is None else args.https_port

    check_config_folder()

    serial_manager = SerialManager()
    try:
        if not serial_manager.is_alive():
            serial_manager.start()
        index_application = tornado.web.Application(
            url_pattern, default_handler_class=My404Handler, **settings)

        index_application.listen(http_port)
        http_server = tornado.httpserver.HTTPServer(
            index_application,
            ssl_options=ssl_opts
        )
        http_server.listen(ssl_port)
        tornado.ioloop.PeriodicCallback(WSHandler.write_to_clients, 500).start()

        terminal_width = get_terminal_size((80, 20))[0]
        introduction(ssl_port, terminal_width)

        tornado.ioloop.IOLoop.instance().start()
    except KeyboardInterrupt:
        print('Exiting...'.center(terminal_width))
    except Exception as e:
        print(e)
        sys.exit()
    finally:
        serial_manager.join()


def introduction(ssl_port, terminal_width):
    """Show a message to the user so he knows who we are."""
    app_name = '  Arduinozore'
    TOPBAR = '#' * (terminal_width - 2)

    print('/' + TOPBAR + '\\' + "\n")
    print(Figlet(font='banner').renderText(app_name))
    print('\\' + TOPBAR + '/' + "\n")
    print('/' + TOPBAR + '\\' + "\n")
    HOST = socket.gethostname()
    PORT = ssl_port
    message = 'Listening on: https://{}:{}'.format(HOST, PORT)
    print(message.center(terminal_width))
    sys.stdout.flush()


def check_config_folder():
    """Check if config folder exists, otherwise creates it."""
    try:
        if not os.path.exists(CONFIG_FOLDER):
            print("No configuration folder found, creating one")
            os.makedirs(CONFIG_FOLDER)
            os.makedirs(CERT_FOLDER)
            os.makedirs(STATIC_PATH)
            os.system(STATIC_CMD)
            os.system(CERT_CMD)
            print("Configuration folder created with success.")

    except Exception as e:
        exit(e)


def copy_arduino_code(dest):
    """Copy arduino source code to dest."""
    dst = path(dest, ARDUINO_CODE_FILE_NAME)
    copyfile(ARDUINO_FILE_PATH, dst)
    print("File copied to " + dst)
    exit(0)


if __name__ == "__main__":
    main()



"""Index page handler package."""

from arduinozore.handlers.crudHandler import CrudHandler
from arduinozore.models.card import Card
from tornado.escape import url_escape


class CardHandler(CrudHandler):
    """Card handler."""

    def list(self):
        """List configurations."""
        self.cards = Card.get_all()
        self.render('card/list.html', cards=self.cards)

    def show(self, slug):
        """Show device."""
        card = Card.get(slug)
        if card is not None:
            self.render('card/show.html', card=card, slug=slug)
        else:
            self.redirect('/card', permanent=False)

    def create(self):
        """Show configuration form for device."""
        settings = dict()
        settings['card'] = None
        settings['method'] = 'post'
        self.render('card/config.html', **settings)

    def edit(self, slug):
        """Show configuration form for device."""
        card = Card.get(slug)
        settings = dict()
        settings['card'] = card
        settings['method'] = 'put'
        self.render('card/config.html', **settings)

    def store(self, slug=""):
        """Create configuration."""
        card_name = self.get_argument('name')
        self.save(card_name)

    def update(self, slug):
        """Update configuration."""
        self.save(slug)

    def save(self, slug):
        """Save configuration."""
        card_name = slug
        nb_input_pins = self.get_argument('nb_input_pins')
        nb_output_pins = self.get_argument('nb_output_pins')
        card = Card(card_name, nb_input_pins, nb_output_pins)
        card.save()
        slug = url_escape(slug)
        redirect_url = self.redirect_url
        if slug not in redirect_url:
            redirect_url += '/' + slug
        self.redirect(redirect_url, permanent=True)

    def destroy(self, slug):
        """Destroy configuration."""
        card = Card.get(slug)
        card.delete()
        self.redirect(self.redirect_url, permanent=False)



"""Index page handler package."""

from arduinozore.handlers.baseHandler import BaseHandler


class SettingPageHandler(BaseHandler):
    """Index page handler."""

    def get(self):
        """Handle get request."""
        available_settings = {'Cartes': 'card',
                              'Capteurs': 'sensor', 'Devices': 'device'}
        self.render('settings.html', available_settings=available_settings)



#!/usr/bin/env python

"""
Pandoc filter to convert all regular text to uppercase.

Code, link URLs, etc. are not affected.
"""

import re

from pandocfilters import Image
from pandocfilters import RawInline
from pandocfilters import toJSONFilter


def rstFilter(key, value, format, meta):
    """
    Filter datas.

    First part search for latex reference to figure and writes them
    as RawInline thereby backslashes are not escaped.

    Second part search for images and set a label in the description
    in order to be able to make a reference to them.
    """
    if key == 'Str':
        if re.search("\$ref", value):
            # if we find a link, set it as raw link and re-add a
            # backslash before the ref keyword
            return RawInline('latex', value[:1] + "\\" + value[1:])
        elif re.search(".*}\$", value):
            return RawInline('latex', value)

    elif key == 'Image':
        [attrs, caption, src] = value
        new_capt = src[0]
        # add \label{IMAGE_LABEL} to the picture description
        new_desc = "\\" + "label{" + new_capt + "}"
        caption.append(RawInline('latex', new_desc))
        return Image(attrs, caption, src)


if __name__ == "__main__":
    toJSONFilter(rstFilter)



"""Logging process."""
import re
import sys
import time
from multiprocessing import Event
from multiprocessing import Manager
from multiprocessing import Process

from arduinozore.models.device import Device
from arduinozore.models.sensor import Sensor
from serial import Serial


class SerialReader(Process):
    """Process class."""

    def __init__(self, serial_port, parent):
        """Init process."""
        Process.__init__(self)
        self.exit = Event()
        self.serial_port = serial_port
        self.device = Device.get(Device.get_identifier_from_serial(serial_port))
        self.get_port_list()
        self.ser = None
        self.parent = parent
        self.parent.set_datas(self.serial_port, dict(self.value))

    def get_port_list(self):
        """Get port list from config."""
        self.ports = Manager().dict()
        self.value = Manager().dict()
        self.sensors = dict()
        for p in self.device.ports['input']:
            if p.enabled:
                self.ports[p.number] = p
                self.value[p.number] = 'Connexion à la carte'
                self.sensors[p.number] = Sensor.get(p._type)

    def get_datas(self):
        """Return datas."""
        return self.value

    def run(self):
        """Run process."""
        self.init_serial()
        while not self.exit.is_set():
            try:
                self.read_serial()
            except (KeyboardInterrupt, RuntimeError) as e:
                self.shutdown()
            except Exception as e:
                exit(e)
            finally:
                try:
                    self.ser.close()
                except AttributeError:
                    pass
        # print("You exited!")

    def shutdown(self):
        """Shut down process."""
        self.exit.set()

    def init_serial(self):
        """Init serial port."""
        # Can't put it in init otherwise makes an error...
        self.ser = Serial(self.serial_port, timeout=None, baudrate=38400)

        self.pattern = re.compile(r'[\t\r\n]+')
        msg2 = 'You can now ask for reading [r] or writing [w].'

        self.ser.read(self.ser.inWaiting())
        datas = re.sub(self.pattern, '', self.ser.readline().decode())
        while datas != msg2:
            self.ser.write("ok".encode())
            time.sleep(0.4)
            datas = re.sub(self.pattern, '', self.ser.readline().decode())
            # print(datas)
            sys.stdout.flush()

    def read_serial(self):
        """Read serial port."""
        try:
            while not self.exit.is_set():
                self.ser.read(self.ser.inWaiting())
                self.get_port_list()
                for port in dict(self.ports):
                    self.ser.read(self.ser.inWaiting()).decode()
                    self.ser.write('r'.encode() + str.encode(str(port)))
                    data = re.sub(self.pattern, '',
                                  self.ser.readline().decode())
                    if self.sensors[port] is not None:
                        self.value[port] = self.sensors[port].transform_datas(
                            data)
                    else:
                        self.value[port] = data
                self.parent.set_datas(self.serial_port, dict(self.value))
                port = self.parent.get_toggelable_pin(self.serial_port)
                if port is not None:
                    self.ser.read(self.ser.inWaiting()).decode()
                    self.ser.write('w'.encode() + str.encode(str(port)))

        except (KeyboardInterrupt) as e:
            exit()



#!/usr/bin/env python

"""
Pandoc filter to convert all regular text to uppercase.

Code, link URLs, etc. are not affected.
"""

import re

from pandocfilters import Image
from pandocfilters import RawInline
from pandocfilters import toJSONFilter


def mdFilter(key, value, format, meta):
    """
    Filter datas.

    First part search for latex reference to figure and writes them
    as RawInline thereby backslashes are not escaped. It also removes
    the extra $ symbols.

    Second part search for images and set a label in the description
    in order to be able to make a reference to them.
    """
    if key == 'Str':
        if re.search("\$ref.*\$", value):
            # if we find a link, set it as raw link and re-add a
            # backslash before the ref keyword and remote
            value = value.replace("$", "")
            val = RawInline('latex', "\\" + value)
            return val

    if key == 'Image':
        [attrs, caption, src] = value
        # this means the image label hasn't been modified yet
        # (the mardown file doesn't come from a rst file)
        if caption[-1]['t'] != 'RawInline':
            new_capt = src[0]
            # add \label{IMAGE_LABEL} to the picture description
            new_desc = "\\" + "label{" + new_capt + "}"
            caption.append(RawInline('latex', new_desc))
            return Image(attrs, caption, src)


if __name__ == "__main__":
    toJSONFilter(mdFilter)
